@@ -19,12 +19,12 @@ import six
 import socket
 from wsgiref import simple_server
 
-from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import netutils
 
 from zaqar.common import decorators
 from zaqar.common.transport.wsgi import helpers
+from zaqar.conf import drivers_transport_wsgi
 from zaqar.i18n import _
 from zaqar import transport
 from zaqar.transport import acl
@@ -37,24 +37,10 @@ from zaqar.transport.wsgi import v1_1
 from zaqar.transport.wsgi import v2_0
 from zaqar.transport.wsgi import version
 
-_WSGI_OPTIONS = (
-    cfg.HostAddressOpt('bind', default='127.0.0.1',
-                       help='Address on which the self-hosting server will '
-                            'listen.'),
-
-    cfg.PortOpt('port', default=8888,
-                help='Port on which the self-hosting server will listen.'),
-)
-
-_WSGI_GROUP = 'drivers:transport:wsgi'
 
 LOG = logging.getLogger(__name__)
 
 
-def _config_options():
-    return [(_WSGI_GROUP, _WSGI_OPTIONS)]
-
-
 class FuncMiddleware(object):
 
     def __init__(self, func):
@@ -69,8 +55,9 @@ class Driver(transport.DriverBase):
     def __init__(self, conf, storage, cache, control):
         super(Driver, self).__init__(conf, storage, cache, control)
 
-        self._conf.register_opts(_WSGI_OPTIONS, group=_WSGI_GROUP)
-        self._wsgi_conf = self._conf[_WSGI_GROUP]
+        self._conf.register_opts(drivers_transport_wsgi.ALL_OPTS,
+                                 group=drivers_transport_wsgi.GROUP_NAME)
+        self._wsgi_conf = self._conf[drivers_transport_wsgi.GROUP_NAME]
         self._validate = validation.Validator(self._conf)
 
         self.app = None
@@ -85,6 +72,14 @@ class Driver(transport.DriverBase):
         return helpers.validate_queue_identification(
             self._validate.queue_identification, req, resp, params)
 
+    def _validate_topic_identification(self, req, resp, params):
+        return helpers.validate_topic_identification(
+            self._validate.topic_identification, req, resp, params)
+
+    def _require_client_id(self, req, resp, params):
+        return helpers.require_client_id(
+            self._validate.client_id_uuid_safe, req, resp, params)
+
     @decorators.lazy_property(write=False)
     def before_hooks(self):
         """Exposed to facilitate unit testing."""
@@ -92,7 +87,7 @@ class Driver(transport.DriverBase):
             self._verify_pre_signed_url,
             helpers.require_content_type_be_non_urlencoded,
             helpers.require_accepts_json,
-            helpers.require_client_id,
+            self._require_client_id,
             helpers.extract_project_id,
 
             # NOTE(jeffrey4l): Depends on the project_id and client_id being
@@ -100,7 +95,10 @@ class Driver(transport.DriverBase):
             helpers.inject_context,
 
             # NOTE(kgriffs): Depends on project_id being extracted, above
-            self._validate_queue_identification
+            self._validate_queue_identification,
+
+            # NOTE(kgriffs): Depends on project_id being extracted, above
+            self._validate_topic_identification
         ]
 
     def _init_routes(self):
@@ -131,6 +129,10 @@ class Driver(transport.DriverBase):
         else:
             self.app = falcon.API(before=self.before_hooks)
 
+        # Set options to keep behavior compatible to pre-2.0.0 falcon
+        self.app.req_options.auto_parse_qs_csv = True
+        self.app.req_options.keep_blank_qs_values = False
+
         self.app.add_error_handler(Exception, self._error_handler)
 
         for version_path, endpoints in catalog:
