@@ -18,7 +18,7 @@ import jsonschema
 from oslo_log import log
 import six
 
-from zaqar.common.api.schemas import flavors as schema
+from zaqar.common.api.schemas.v1_1 import flavors as schema
 from zaqar.common import utils as common_utils
 from zaqar.i18n import _
 from zaqar.storage import errors
