@@ -42,14 +42,11 @@ from oslo_log import log
 import six
 
 from zaqar.common.api.schemas import pools as schema
-from zaqar.common import decorators
 from zaqar.common import utils as common_utils
 from zaqar.i18n import _
 from zaqar.storage import errors
 from zaqar.storage import utils as storage_utils
-from zaqar.transport import acl
 from zaqar.transport import utils as transport_utils
-from zaqar.transport import validation
 from zaqar.transport.wsgi import errors as wsgi_errors
 from zaqar.transport.wsgi import utils as wsgi_utils
 
@@ -62,12 +59,9 @@ class Listing(object):
     :param pools_controller: means to interact with storage
     """
 
-    def __init__(self, pools_controller, validate):
+    def __init__(self, pools_controller):
         self._ctrl = pools_controller
-        self._validate = validate
 
-    @decorators.TransportLog("Pools collection")
-    @acl.enforce("pools:get_all")
     def on_get(self, request, response, project_id):
         """Returns a pool listing as objects embedded in an object:
 
@@ -93,12 +87,6 @@ class Listing(object):
         request.get_param_as_int('limit', store=store)
         request.get_param_as_bool('detailed', store=store)
 
-        try:
-            self._validate.pool_listing(**store)
-        except validation.ValidationFailed as ex:
-            LOG.debug(ex)
-            raise wsgi_errors.HTTPBadRequestAPI(six.text_type(ex))
-
         cursor = self._ctrl.list(**store)
         pools = list(next(cursor))
 
@@ -136,14 +124,11 @@ class Resource(object):
         self._validators = {
             'weight': validator_type(schema.patch_weight),
             'uri': validator_type(schema.patch_uri),
-            'group': validator_type(schema.patch_group),
-            'flavor': validator_type(schema.patch_flavor),
+            'group': validator_type(schema.patch_uri),
             'options': validator_type(schema.patch_options),
             'create': validator_type(schema.create)
         }
 
-    @decorators.TransportLog("Pools item")
-    @acl.enforce("pools:get")
     def on_get(self, request, response, project_id, pool):
         """Returns a JSON object for a single pool entry:
 
@@ -169,8 +154,6 @@ class Resource(object):
 
         response.body = transport_utils.to_json(data)
 
-    @decorators.TransportLog("Pools item")
-    @acl.enforce("pools:create")
     def on_put(self, request, response, project_id, pool):
         """Registers a new pool. Expects the following input:
 
@@ -195,8 +178,6 @@ class Resource(object):
         try:
             self._ctrl.create(pool, weight=data['weight'],
                               uri=data['uri'],
-                              group=data.get('group', None),
-                              flavor=data.get('flavor', None),
                               options=data.get('options', {}))
             response.status = falcon.HTTP_201
             response.location = request.path
@@ -208,8 +189,6 @@ class Resource(object):
             LOG.exception(e)
             raise wsgi_errors.HTTPConflict(six.text_type(e))
 
-    @decorators.TransportLog("Pools item")
-    @acl.enforce("pools:delete")
     def on_delete(self, request, response, project_id, pool):
         """Deregisters a pool.
 
@@ -230,15 +209,13 @@ class Resource(object):
 
         response.status = falcon.HTTP_204
 
-    @decorators.TransportLog("Pools item")
-    @acl.enforce("pools:update")
     def on_patch(self, request, response, project_id, pool):
         """Allows one to update a pool's weight, uri, and/or options.
 
         This method expects the user to submit a JSON object
-        containing at least one of: 'uri', 'weight', 'group', 'flavor',
-        'options'.If none are found, the request is flagged as bad.
-        There is also strict format checking through the use of
+        containing at least one of: 'uri', 'weight', 'group', 'options'. If
+        none are found, the request is flagged as bad. There is also
+        strict format checking through the use of
         jsonschema. Appropriate errors are returned in each case for
         badly formatted input.
 
@@ -248,12 +225,11 @@ class Resource(object):
         LOG.debug(u'PATCH pool - name: %s', pool)
         data = wsgi_utils.load(request)
 
-        EXPECT = ('weight', 'uri', 'group', 'flavor', 'options')
+        EXPECT = ('weight', 'uri', 'options')
         if not any([(field in data) for field in EXPECT]):
             LOG.debug(u'PATCH pool, bad params')
             raise wsgi_errors.HTTPBadRequestBody(
-                'One of `uri`, `weight`, `group`, `flavor`,'
-                ' or `options` needs '
+                'One of `uri`, `weight`,or `options` needs '
                 'to be specified'
             )
 
@@ -268,13 +244,9 @@ class Resource(object):
             )
         fields = common_utils.fields(data, EXPECT,
                                      pred=lambda v: v is not None)
-        resp_data = None
+
         try:
             self._ctrl.update(pool, **fields)
-            resp_data = self._ctrl.get(pool, False)
         except errors.PoolDoesNotExist as ex:
             LOG.exception(ex)
             raise wsgi_errors.HTTPNotFound(six.text_type(ex))
-
-        resp_data['href'] = request.path
-        response.body = transport_utils.to_json(resp_data)
